import datetime
import gettext
import sys
import time
import tkinter
import tkinter.ttk as ttk
from tkinter.filedialog import askopenfilename


class ToolBar(ttk.Frame):
    """
        ToolBar that sits below the MenuBar.
    """
    def __init__(self, parent):
        """
            Provides the option for gcal_to_xlsx or pdf_to_json
        """
        ttk.Frame.__init__(self, parent)

        # Tool 1: gcal_to_xlsx
        gcal_tool = ttk.Button(self,
                               text="Google Calendar to Excel",
                               command=self.gcal_to_xlsx)

        # Tool 2: pdf_to_json
        pdf_tool = ttk.Button(self,
                              text="PDF to JSON",
                              command=self.pdf_to_json)

        self.buttons = [x.pack(side=tkinter.LEFT, fill=tkinter.X)
                        for x in [gcal_tool, pdf_tool]]

        self.config(border=1, relief=tkinter.GROOVE)
        self.pack()

    def gcal_to_xlsx(i):
        name = tkinter.filedialog.askopenfilename()
        if isinstance(name, str):
            print('File selected for open: ' + name)
        else:
            print('No file selected')

    def pdf_to_json(i):
        name = tkinter.filedialog.askopenfilename()
        if isinstance(name, str):
            print('File selected for open: ' + name)
        else:
            print('No file selected')



import re

CLASS_CATEGORIES = [
    "Master Class",
    "Brunch Bunch",
    "Rush Hour Group",
    "Full Bronze Group",
    "All-Level Group",
    "Practice Session",
    "Dance Series",
    "Bronze I & II Group",
    "Encore",
    "Dance Fundamentals",
    "Wedding Group",
    "Amalgamation Group",
    "Formation",
    "Advanced Practice Session",
    "Ice Cream Social"
]

CATS_WITH_STYLE = [
    x for x in CLASS_CATEGORIES
    if x not in ["Ice Cream Social",
                 "Encore",
                 "Formation",
                 "Wedding Group",
                 "Practice Session",
                 "Advanced Practice Session",
                 "Master Class"]
]

DANCE_STYLES = [
    "Argentine Tango",
    "Bachata",
    "Bolero",
    "Cha Cha",
    "Foxtrot",
    "Hustle",
    "Lindy Hop",
    "Line Dancing",
    "Merengue",
    "Quickstep",
    "Rumba",
    "Salsa",
    "Samba",
    "Swing",
    "Tango",
    "Country Two Step",
    "Viennese Waltz",
    "Waltz",
    "West Coast Swing",
    "Zouk",
    "Tango/Swing",
    "Swing/Hustle",
    "Rumba/Hustle",
    "Foxtrot/Hustle",
    "Rumba/Merengue",
    "Waltz/Cha Cha"
]

REGEXES = {
    # Find out if it's a single time or time range
    "timestamp": re.compile(r'\d+(am|pm)' +
                            r'|\d+(am|pm)?-(\d+|\d+:\d\d)(am|pm)' +
                            r'|\d+:\d\d(am|pm)?(-(\d+:\d\d|\d+)(am|pm)?)?'),

    "CLASS_CATEGORIES": [
        re.compile(r'masterclass'),
        re.compile(r'brunchbunch'),
        re.compile(r'rushhourgroup'),
        re.compile(r'fullbronzegroup'),
        re.compile(r'all-levelgroup'),
        re.compile(r'^practicesession$'),
        re.compile(r'danceseries'),
        re.compile(r'bronzei&iigroup'),
        re.compile(r'encore'),
        re.compile(r'dancefundamentals'),
        re.compile(r'weddinggroup'),
        re.compile(r'^amalgamation'),
        re.compile(r'formation'),
        re.compile(r'^advanced'),
        re.compile(r'^(icecream)?(social)')
    ],
    "DANCE_STYLES": [
        re.compile(r'argentinetango'),
        re.compile(r'bachata'),
        re.compile(r'bolero'),
        re.compile(r'chacha'),
        re.compile(r'foxtrot'),
        re.compile(r'hustle'),
        re.compile(r'lindyhop'),
        re.compile(r'linedancing'),
        re.compile(r'merengue'),
        re.compile(r'quickstep'),
        re.compile(r'rumba'),
        re.compile(r'salsa'),
        re.compile(r'samba'),
        re.compile(r'swing'),
        re.compile(r'tango'),
        re.compile(r'countrytwostep'),
        re.compile(r'viennesewaltz'),
        re.compile(r'waltz'),
        re.compile(r'westcoastswing'),
        re.compile(r'zouk'),
        re.compile(r'foxtrot\/hustle'),
        re.compile(r'tango\/swing'),
        re.compile(r'swing\/hustle'),
        re.compile(r'rumba\/hustle'),
        re.compile(r'rumba\/merengue'),
        re.compile(r'waltz\/chacha')
    ]
}



"""gcal_to_xlsx: Converting Google Calendar calendars to XLSX

<month_and_year> as well as <google_calendar_name> need to be surrounded
by quotes.

Usage:
    amvernon_gcal_to_xlsx [options] <month_and_year>

Arguments:
    client_secret.json                      Name of client secret (in quotes)
    google_calendar_name                    Name of calendar in Google Calendar (in quotes)
    <month_and_year>                        Example: "September 2017"

Options:
    --client_secret=client_secret.json      Specify location of client_secret [default: client_secret.json]
    --calendar=google_calendar_name         Specify name of calendar. [default: Arthur Murray Vernon Calendar]
    -h --help                               Show this screen.
    --version                               Show version.

"""
import os
import maya
import json
import pathlib
import datetime
import calendar
import httplib2
import xlsxwriter
import argparse
import pandas as pd
from docopt import docopt
from apiclient import discovery
from oauth2client import client
from oauth2client import tools
from oauth2client.file import Storage


__version__ = "0.1.0"
__author__ = "Vincent Chov"
__license__ = "MIT"


def get_credentials(CLIENT_SECRET_FILE='client_secret.json', flags=None):
    """Gets valid user credentials from storage.

    If nothing has been stored, or if the stored credentials are invalid,
    the OAuth2 flow is completed to obtain the new credentials.

    Returns:
        Credentials, the obtained credential.
    """
    if not os.path.isfile(CLIENT_SECRET_FILE):
        raise IOError("{} not found...".format(CLIENT_SECRET_FILE))
    SCOPES = 'https://www.googleapis.com/auth/calendar.readonly'
    APPLICATION_NAME = 'Arthur Murray Vernon Calendar'
    home_dir = os.path.expanduser('~')
    credential_dir = os.path.join(home_dir, '.credentials')
    if not os.path.exists(credential_dir):
        os.makedirs(credential_dir)
    credential_path = os.path.join(credential_dir,
                                   'amvernoncal_credentials.json')

    store = Storage(credential_path)
    credentials = store.get()
    if not credentials or credentials.invalid:
        flow = client.flow_from_clientsecrets(CLIENT_SECRET_FILE, SCOPES)
        flow.user_agent = APPLICATION_NAME
        print(type(flow))
        if flags:
            credentials = tools.run_flow(flow, store, flags)
        else:
            credentials = tools.run_flow(flow, store)
        print('Storing credentials to ' + credential_path)
    return credentials


def get_gcal_by_summary(desired_month_string, summary, service):
    """
        Gets all events from Google Calendar for the current month where the
        calendar's name matches the inputted one.
    """
    # Get the calendarId for the inputted calendar name
    cals = service.calendarList().list().execute()['items']
    calendars = [x for x in cals if summary.lower() in x['summary'].lower()][0]

    cal_id = calendars['id']

    # Get the first and last days of the month
    desired_month = maya.when(desired_month_string)
    first_day = (desired_month.subtract(days=desired_month.day-1)
                              .datetime(to_timezone='US/Eastern').isoformat())
    next_month_string = '{} {}'.format(desired_month.month + 1,
                                       desired_month.year)
    next_month = maya.when(next_month_string)
    last_day = (next_month.subtract(days=next_month.day-1)
                          .datetime(to_timezone='US/Eastern').isoformat())

    # Get all events for this month for the selected calendar
    events = service.events().list(calendarId=cal_id, singleEvents=True,
                                   timeMin=first_day, timeMax=last_day,
                                   orderBy='startTime').execute()

    return events['items']


def format_gcal_to_json(events, raw_filename=''):
    """
        Condenses the relevant info from the Google Calendar events down to be
        closer to the JSON structure we'd get if we went from PDF to JSON.
    """
    fmtd_events = []
    for event in events:
        index = len(fmtd_events) - 1

        date_time = (maya.parse(event['start']['dateTime'])
                         .datetime(to_timezone='US/Eastern'))
        date = date_time.strftime('%d').lstrip('0')

        # Get or create an object for the current date
        obj_exists = len(fmtd_events) != 0 and date in fmtd_events[-1]['date']
        if obj_exists:
            obj = fmtd_events[-1]
        else:
            obj = {'date': date, 'events': []}

        # Event Category/Title
        category = event['summary']

        # Get the timestamp for this event only if the studio is open right now
        if 'Studio Closed'.lower() not in event['summary'].lower():
            time = date_time.strftime('%I:%M%p').lstrip('0') + '\n'
        else:
            time = ''

        event_obj = {
            'Event': category,
            'Time': time
        }

        description_str = event.get('description', '')
        has_style = 'style' in description_str.lower()
        descriptions = description_str.splitlines()
        if has_style:
            style = descriptions[0].replace('Style: ', '')
            if len(descriptions) > 1:
                event_obj['Description'] = description
            else:
                style += ' '
        else:
            if descriptions:
                style = '\n '
                description = '\n'.join(descriptions) + '\n'
                obj['Description'] = description
            else:
                style = ''

        event_obj['Style'] = style
        # Add the event object to the current day's object
        obj['events'].append(event_obj)

        # Either modify the existing event in-place or add a new one
        if obj_exists:
            fmtd_events[index] = obj
        else:
            fmtd_events.append(obj)

    filename = "Output/{}.json".format(raw_filename)
    with open(filename, 'w') as json_file:
        json_file.write(json.dumps(fmtd_events, indent=4, sort_keys=True))
    return fmtd_events


def generate_cal_strings(day_json, workbook):
    """
        For a given day's JSON object, generate string of the structure that
        the printable PDF would use, with xlsxwriter formats.
    """
    bold_italics_fmt = workbook.add_format({
        'bold': True,
        'italic': True,
        'font_name': 'Times New Roman'
    })
    italics_fmt = workbook.add_format({
        'italic': True,
        'font_name': 'Times New Roman'
    })

    events_list = day_json['events']
    event_strings = []
    for e in events_list:
        event_strings += [bold_italics_fmt, e['Event'] + '\n']
        event_strings += [italics_fmt, e['Style']]
        event_strings += [bold_italics_fmt, e.get('Description', '')]
        event_strings += [italics_fmt, '{}\n'.format(e['Time'])]

    event_strings[-1] += '\n'

    day_str = (
        [bold_italics_fmt, day_json['date'] + '\n\n'] +
        event_strings +
        [bold_italics_fmt, day_json.get('Description', '')]
    )
    return day_str


def gcal_events_to_xlsx(desired_month_string, calendar_title, service):
    """
        For a given month, put all of the Google Calendar events into a JSON
        and then output an XLSX file with the JSON's data formatted.
    """
    events = [
        e for e in
        get_gcal_by_summary(desired_month_string, calendar_title, service)
        if 'dance lesson' not in e['summary'].lower()
    ]

    filename = desired_month_string.lower().replace(' ', '_')
    fmtd_events = format_gcal_to_json(events, filename)

    # ints arranged like a calendar's days of the week
    month = maya.when(desired_month_string)
    month_cal = (calendar.TextCalendar(calendar.SUNDAY)
                         .monthdayscalendar(month.year, month.month))

    # Create the XLSX writer and set the font format
    filename = desired_month_string.lower().replace(' ', '_')
    workbook = xlsxwriter.Workbook('Output/{}.xlsx'.format(filename))
    worksheet = workbook.add_worksheet()

    # Add on the event strings to each day in the calendar
    for i, week_cell in enumerate(month_cal):
        for j, day_cell in enumerate(week_cell):
            if day_cell != 0 and day_cell <= len(fmtd_events):
                day_json = fmtd_events[day_cell-1]
                day_strings = generate_cal_strings(day_json, workbook)
                worksheet.write_rich_string(i, j, *day_strings)
    print('Done!')
    return workbook


def main():
    args = docopt(__doc__, version=__version__)

    client_secret_path = args.get('--client_secret', None)
    month_year = args['<month_and_year>']
    calendar_name = args.get('--calendar', None)
    
    # Prompt user for permission using argparse if we don't already have it
    try:
        flags = tools.argparser.parse_args([])
    except:
        flags = None

    if not client_secret_path:
        credentials = get_credentials(flags=flags)
    else:
        credentials = get_credentials(CLIENT_SECRET_FILE=client_secret_path,
                                      flags=flags)
    http = credentials.authorize(httplib2.Http())
    service = discovery.build('calendar', 'v3', http=http)

    pathlib.Path('JSONs').mkdir(parents=True, exist_ok=True)
    pathlib.Path('PDFs').mkdir(parents=True, exist_ok=True)
    pathlib.Path('Output').mkdir(parents=True, exist_ok=True)

    if not calendar_name:
        calendar_name = "Arthur Murray Vernon Calendar"
    
    try:
        book = gcal_events_to_xlsx(month_year, calendar_name, service)
        book.close()
    except IndexError as e:
        raise ValueError("Calendar not found or no events found") from e


if __name__ == '__main__':
    main()



import json
import re
from fuzzywuzzy import fuzz, process
from ..classification.constants_and_regexes import (REGEXES, DANCE_STYLES,
                                                    CLASS_CATEGORIES)
from ..classification.classifier import (cat_or_style_cl, cat_cl,
                                         getStylesExact, getClassesExact)
from .process_cal import (split_on_timestamps, group_event_strings,
                         generate_events)


def pre_clean_calendar(json_data):
    cal_data = []

    for list_of_objects in json_data:
        for obj in list_of_objects:
            try:
                assert 'text' in obj.keys()
                valid_obj = int(obj['text'][0])
                obj.pop('height', None)
                obj.pop('width', None)
                obj.pop('left', None)
                obj.pop('top', None)

                # Remove whitespace between dates and strings
                obj['text'] = re.sub(r'(?!\d+)\s\s+(?=\w)', '\r', obj['text'])
                obj['text'] = re.sub(r'\r\(', ' (', obj['text'])
                # Split the raw text by the carriage returns
                obj['text'] = re.split(r'\r', obj['text'])

                # Try to split on timestamps
                split_text, timestamps = split_on_timestamps(obj['text'])
                # Add date
                obj['date'] = split_text.pop(0)
                obj['text'] = split_text
                obj['timestamps'] = timestamps

                cal_data.append(obj)
            except (ValueError, IndexError):
                continue
    return cal_data


def post_clean_calendar(cal_data):
    # Fill cal_data with JSON objects that may contain dates
    for obj in cal_data:
        try:
            text_copy = obj["text"]

            if 'timestamps' in obj.keys():
                timestamps = obj['timestamps']
            else:
                timestamps = []

            # Group together related strings if there are timestamps
            if timestamps:
                obj['text'] = group_event_strings(text_copy, timestamps)
            elif "Studio Closed" in obj['text']:
                obj['text'] = group_event_strings(text_copy, [])
            else:
                # If there are no timestamps, ie, if the text says
                # 'Studio Closed' w/out a time, pass it right through
                obj['text'] = text_copy

            obj['events'] = generate_events(obj['text'])
            obj.pop('text', None)
            obj.pop('timestamps', None)
        except (ValueError, IndexError):
            continue

    # Assert that we have a full calendar
    if (len(cal_data) < 28) or (len(cal_data) > 31):
        raise SystemExit("Error: We don't have a full calendar...")
    else:
        print("Yay! We have all the calendar events!")
        return cal_data
    return cal_data


def separate_clumped_events(text_list, timestamps):
    triples = zip(text_list, text_list[1:], text_list[2:])

    i = 0
    for x, y, z in triples:
        x_type = cat_or_style_cl.classify(x)
        y_type = cat_or_style_cl.classify(y)
        if all([x_type == 'cat', y_type == 'dance_style', z in timestamps]):
            break
        else:
            i += 1

    return [text_list[:i], text_list[i:i+3], text_list[i+3:]]



import json
import re
from fuzzywuzzy import fuzz, process
from ..classification.constants_and_regexes import (REGEXES, DANCE_STYLES,
                                                    CLASS_CATEGORIES)
from ..classification.classifier import (cat_or_style_cl, cat_cl, 
                                         determine_structure, getClassesExact,
                                         getStylesExact)


def generate_events(events_lists):
    # TODO: generate events based on what we're told is the event structure,
    # not just assuming from the length
    event_objects = []
    keys_to_map = []
    for events_strings in events_lists:
        if len(events_strings) == 3:
            keys_to_map = ['Event', 'Style', 'Time']
        elif len(events_strings) == 2:
            # Find out if the other element is a style or timestamp
            if REGEXES['timestamp'].search(events_strings[1]):
                keys_to_map = ['Event', 'Time']
            else:
                keys_to_map = ['Event', 'Style']
        else:
            keys_to_map = ['Event']

        event_object = dict(zip(keys_to_map, events_strings))
        event_objects.append(event_object)

    return event_objects


def merge_strings(text_list):
    # Try all possible pairs of strings until we get a pair that
    # produce the same match
    pairs = zip(text_list, text_list[1:])

    for x, y in pairs:
        x_type = cat_or_style_cl.classify(x)
        y_type = cat_or_style_cl.classify(y)

        # Return a match if x+y gets the same match as just x
        if x_type != y_type:
            # Assert that we x and y are the same time
            continue

        # Generate potential matches for both x and xy
        xy = "{} {}".format(x, y)
        if x_type == "cat":
            xy_type = cat_cl.classify(xy)
            potential_matches = getClassesExact([x, xy])
        elif x_type == "dance_style":
            potential_matches = getStylesExact([x, xy])

        potential_match_x = potential_matches[0][0]
        potential_match_xy = potential_matches[0][1]
        if potential_match_x.upper() in potential_match_xy.upper():
            match = potential_match_xy

        x_index = text_list.index(x)
        y_index = x_index + 1
        text_copy = text_list[:x_index] + [match] + text_list[y_index + 1:]
        # Return the match as well as the element to replace
        return text_copy


def group_event_strings(text_list, timestamps):
    text_list = [x for x in text_list if x != ""]
    groups = []
    start = 0

    for index, string in enumerate(text_list):
        if string in timestamps:
            # Add all the string up to and including the timestamp to a list
            group = text_list[start:index + 1]
            # Add that list grouping together strings to text_copy
            groups.append(group)
            start = index + 1
        elif string == "Studio Closed":
            groups.append(text_list)
            break

    for index, group in enumerate(groups):
        # Merge classes and styles that are spread over two lines
        structure = determine_structure(group, timestamps)
        if structure == "SPREAD":
            groups[index] = merge_strings(group)
        elif structure == "CLOSING":
            groups[index] = ["Studio Closed"]
        elif structure == "CLOSING_DOUBLE":
            groups[index] = ["Studio Closed: {}".format(group[1])]
        elif structure == "CLOSING_SPREAD_TIME":
            groups[index] = ["Studio Closed: {}".format(group[1]), group[2]]
        elif structure == "CLOSING_SPREAD":
            groups[index] = ["Studio Closed: {}".format(group[1])] + group[2:]
        elif structure == "CAT_AND_TIME":
            groups[index] = [group[0], group[1]]
        elif structure == "CAT_AND_STYLE":
            groups[index] = [group[0], group[1]]
        else:
            groups[index] = group

    return groups


def split_on_timestamps(text_list):
    # Get all the timestamps for the current date_object
    timestamps = [
        x.group(0) for x in
        list(map(lambda y: REGEXES['timestamp'].search(y), text_list)) if x
    ]

    if not timestamps:
        return (text_list, [])
    else:
        text_copy = []
        for string in text_list:
            text_copy.append(string)
            for timestamp in timestamps:
                if timestamp in string:
                    text_copy[-1] = string.replace(timestamp, '')
                    text_copy.append(timestamp)
                    break
        text_copy = [x.strip() for x in text_copy]
        return (text_copy, timestamps)



import datetime
import gettext
import sys
import time
import tkinter
import tkinter.ttk as ttk
from tkinter.filedialog import askopenfilename
from dialogs import PopupDialog
from toolbar import ToolBar
from menubar import MenuBar


class MainFrame(ttk.Frame):
    """
        It will take a while to get into the mainframe.
    """

    past_time = datetime.datetime.now()
    _product = 'Arthur Murray Vernon Calendar\n\n'

    def __init__(self, parent):
        """
            We're in!
        """
        ttk.Frame.__init__(self, parent)
        self.display = ttk.Label(parent, anchor=tkinter.CENTER,
                                 foreground='green', background='black')
        self.display.pack(fill=tkinter.BOTH, expand=1)
        self.tick()

    def tick(self):
        """
            Automatically updates a clock
        """

        this_time = datetime.datetime.now()
        if this_time != self.past_time:
            self.past_time = this_time
            _timestamp = this_time.strftime('%Y-%m-%d %H:%M:%S')
            self.display.config(text=self._product + _timestamp)
        self.display.after(100, self.tick)


class Application(tkinter.Tk):
    """
        Top-level Tkinter widget.
    """

    def __init__(self):
        tkinter.Tk.__init__(self)
        menubar = MenuBar(self)
        self.config(menu=menubar)
        self.wm_title('Arthur Murray Vernon Calendar')
        self.wm_geometry('640x480')

        # Tool bar selection == 'y'
        self.toolbar = ToolBar(self)
        self.toolbar.pack(side='top', fill='x')

        self.mainframe = MainFrame(self)
        self.mainframe.pack(side='right', fill='y')


if __name__ == '__main__':
    APPLICATION_GUI = Application()
    APPLICATION_GUI.mainloop()



import datetime
import gettext
import sys
import time
import tkinter
import tkinter.ttk as ttk
from tkinter.filedialog import askopenfilename


class PopupDialog(ttk.Frame):
    """
        Generic PopupDialog
    """

    def __init__(self, parent, title, body):
        ttk.Frame.__init__(self, parent)
        self.top = tkinter.Toplevel(parent)
        _label = ttk.Label(self.top, text=body, justify=tkinter.LEFT)
        _label.pack(padx=10, pady=10)
        _button = ttk.Button(self.top, text="OK", command=self.ok_button)
        _button.pack(pady=5)
        self.top.title(title)

    def ok_button(self):
        "OK button feedback."

        self.top.destroy()



"""pdf_to_json: Converting PDF calendars to JSON

Usage:
  pdf_to_json  <calendar_name.pdf>

Options:
  -h --help         Show this screen.
  --to-file=<t/f>   Enter true or false to save the file [default: True]
  --version         Show version.

"""
import re
import os
import json
import pathlib
import subprocess
from fuzzywuzzy import fuzz, process
from ..classification.constants_and_regexes import (REGEXES, DANCE_STYLES,
                                                    CLASS_CATEGORIES)
from ..jsonproc.process_cal import split_on_timestamps, group_event_strings
from ..jsonproc.cal_cleaner import pre_clean_calendar, post_clean_calendar

__version__ = "0.1.0"
__author__ = "Vincent Chov"
__license__ = "MIT"


def parse_calendar(json_filename, to_file=False):
    raw_filename = json_filename.replace('.json', '')
    cal = tabula_pdf_to_raw_json(raw_filename)
    assert type(cal[0]['data']) is list
    raw_data = cal[0]['data']
    pre_cleaned = pre_clean_calendar(raw_data)
    cal_data = post_clean_calendar(pre_cleaned)

    input_path = 'JSONs/{}.json'.format(raw_filename)
    output_path = 'Output/{}.json'.format(raw_filename)

    if to_file is True:
        with open(output_path, 'w') as output_file:
            output_file.write(json.dumps(cal_data, indent=4, sort_keys=True))
    else:
        print(json.dumps(cal_data, indent=4, sort_keys=True))
        return cal_data


def tabula_pdf_to_raw_json(pdf_filename):
    """
        Takes a PDF filename with or without its extension and returns a JSON
        outputted by tabula-cli.jar that's also saved to the JSONs folder.
    """
    # Write the input/output paths using the filename
    # Remove the PDF extension if it was accidentally supplied
    pathlib.Path('JSONs').mkdir(parents=True, exist_ok=True)
    pathlib.Path('PDFs').mkdir(parents=True, exist_ok=True)
    pathlib.Path('Output').mkdir(parents=True, exist_ok=True)
    raw_filename = pdf_filename.replace('.pdf', '')
    input_path = 'PDFs/{}.pdf'.format(raw_filename)
    output_path = 'JSONs/{}.json'.format(raw_filename)

    # Guess the area of the table, extract with lattice mode, output JSON
    options = ['java', '-jar', 'tabula-cli.jar', input_path, '--guess',
               '--lattice', '--format', 'JSON', '--outfile', output_path]

    # Wait until the command to convert the PDF to JSON to load and return it
    conversion_proc = subprocess.Popen(options)
    conversion_proc.wait()
    with open(output_path) as json_file:
        converted_json = json.load(json_file)

    return converted_json


if __name__ == '__main__':
    args = docopt(__doc__, version=__version__)
    pdf_path = args["<calendar_name.pdf>"]
    to_file = 'true' in args["to_file"].lower()
    converted_json = parse_calendar(pdf_path, to_file=to_file)



import datetime
import gettext
import sys
import time
import tkinter
import tkinter.ttk as ttk
from tkinter.filedialog import askopenfilename
from dialogs import PopupDialog

class MenuBar(tkinter.Menu):
    """
        MenuBar with "File" and "Help" drop-downs
    """

    def __init__(self, parent):
        tkinter.Menu.__init__(self, parent)

        # File Menu
        filemenu = tkinter.Menu(self, tearoff=False)
        filemenu.add_command(label='Exit',
                             underline=1,
                             command=self.quit)

        # Help Menu
        helpmenu = tkinter.Menu(self, tearoff=False)
        helpmenu.add_command(label='Help',
                             command=lambda x=None: self.help_dialog('Help.'),
                             accelerator="F1")
        helpmenu.add_command(label='About', command=self.about_dialog)
        self.bind_all('<F1>', self.help_dialog)

        self.add_cascade(label='File', underline=0, menu=filemenu)
        self.add_cascade(label='Help', underline=0, menu=helpmenu)

    def quit(self):
        sys.exit(0)

    def help_dialog(self, event):
        _description = """
            Help section not created yet.  Wah wah wah...
        """
        PopupDialog(self, 'Arthur Murray Vernon Calendar', _description)

    def about_dialog(self):
        _description = """
            A Tkinter GUI for amvernoncal

            Author: Vincent Chov
            Email: vincent.chov@uconn.edu
            Version: 0.0.1
            GitHub Package : amvernoncal
        """
        PopupDialog(self, 'About', _description)





import json
import re
from fuzzywuzzy import fuzz, process
from textblob.classifiers import NaiveBayesClassifier
from textblob.classifiers import PositiveNaiveBayesClassifier
from .constants_and_regexes import REGEXES, DANCE_STYLES, CLASS_CATEGORIES


def determine_structure(group, timestamps=[]):
    length = len(group)
    types = []
    for i in group:
        if i in timestamps:
            types.append("timestamp")
        elif "Studio Closed" in i:
            types.append("closing")
        else:
            i_type = cat_or_style_cl.classify(i)
            if i_type == "cat":
                i_type = cat_cl.classify(i)
            types.append(i_type)
    if "closing" in types:
        if length == 1:
            return "CLOSING"
        elif length == 2:
            return "CLOSING_DOUBLE"
        elif length > 2 and group[2] in timestamps:
            return "CLOSING_SPREAD_TIME"
        else:
            return "CLOSING_SPREAD"
    elif length == 2:
        if "cat" in types[0]:
            if types[1] == "timestamp":
                return "CAT_AND_TIME"
            elif types[1] == "dance_style":
                return "CAT_AND_STYLE"
        elif types[0] == "dance_style":
            if types[1] == "timestamp":
                return "STYLE_AND_TIME"
            else:
                return "STYLE AND UNKNOWN"
    elif length == 3:
        if ("cat" in types[0]
                and types[1] == "dance_style"
                and types[2] == "timestamp"):
            return "CAT_STYLE_TIME"
        else:
            return "SPREAD"
    elif length % 2 == 0 and length > 2:
        return "SPREAD"
    elif length % 2 == 1 and length > 3:
        return "CLUMPED"
    else:
        return "OTHER"


def getStylesExact(text):
    style_matches = []
    text_copy = text
    indices_to_drop = []
    for i, original_string in enumerate(text_copy):
        string = original_string.lower().replace(' ', '')
        matches = [x.group() for x in list(map(lambda y: y.search(string),
                   REGEXES["DANCE_STYLES"])) if x and x.group() == string]
        if matches:
            match = process.extractOne(matches[0], DANCE_STYLES)[0]
            style_matches.append(match)
        indices_to_drop.append(i)

    text = [x for i, x in enumerate(text) if i not in indices_to_drop]
    return (style_matches, text)


def getClassesExact(text):
    class_matches = []
    text_copy = text
    indices_to_drop = []
    for i, original_string in enumerate(text_copy):
        string = original_string.lower().replace(' ', '')
        matches = [x.group() for x in list(map(lambda y: y.search(string),
                   REGEXES["CLASS_CATEGORIES"])) if x]
        if matches:
            match = process.extractOne(matches[0], CLASS_CATEGORIES)[0]
            class_matches.append(match)
            indices_to_drop.append(i)
        else:
            class_matches.append(original_string)
            indices_to_drop.append(i)
        indices_to_drop.append(i)
    text = [x for i, x in enumerate(text) if i not in indices_to_drop]
    return (class_matches, text)


def custom_extractor(document):
    tokens = document.split()
    first_word, last_word = tokens[0], tokens[-1]
    feats = {}
    feats["first({0})".format(first_word)] = True
    feats["last({0})".format(last_word)] = True
    feats["first_upper({0})".format(first_word.upper())] = True
    feats["last_upper({0})".format(last_word.upper())] = True
    feats["num_tokens({0})".format(len(tokens))] = True
    has_slash = "/" in document
    feats["has_slash({0})".format(has_slash)] = True
    has_closing = "Studio Closed" in document
    feats["has_closing({0})".format(has_closing)] = True
    return feats


cat_or_style_trainer = [
    ("BRUNCH BUNCH", "cat"),
    ("RUSH HOUR GROUP", "cat"),
    ("FULL BRONZE GROUP", "cat"),
    ("ALL LEVEL GROUP", "cat"),
    ("DANCE SERIES", "cat"),
    ("BRONZE I & II GROUP", "cat"),
    ("DANCE FUNDAMENTALS", "cat"),
    ("Amalgamation GROUP", "cat"),
    ("ICE CREAM SOCIAL", "cat"),
    ("ENCORE", "cat"),
    ("FORMATION", "cat"),
    ("WEDDING GROUP", "cat"),
    ("PRACTICE SESSION", "cat"),
    ("ADVANCED PRACTICE SESSION", "cat"),
    ("MASTER CLASS", "cat"),
    ("MARTIN DONTIGNY COACHING", "cat"),
    ("PATTY CONTENTA COACHING", "cat"),
    ("BACHATA", "dance_style"),
    ("BOLERO", "dance_style"),
    ("CHA CHA", "dance_style"),
    ("FOXTROT", "dance_style"),
    ("HUSTLE", "dance_style"),
    ("LINDY HOP", "dance_style"),
    ("MERENGUE", "dance_style"),
    ("QUICKSTEP", "dance_style"),
    ("RUMBA", "dance_style"),
    ("SALSA", "dance_style"),
    ("SAMBA", "dance_style"),
    ("SWING", "dance_style"),
    ("TANGO", "dance_style"),
    ("WALTZ", "dance_style"),
    ("WEST COAST SWING", "dance_style"),
    ("TANGO/SWING", "dance_style"),
    ("FOXTROT/HUSTLE", "dance_style"),
    ("WALTZ/CHA CHA", "dance_style"),
    ("Studio Closed", "closing"),
    ("STUDIO CLOSED", "closing"),
    ("Practice Party", "cat")
]

cat_trainer = [
    ("BRUNCH BUNCH", "cat_w_style"),
    ("RUSH HOUR GROUP", "cat_w_style"),
    ("FULL BRONZE GROUP", "cat_w_style"),
    ("ALL LEVEL GROUP", "cat_w_style"),
    ("DANCE SERIES", "cat_w_style"),
    ("BRONZE I & II GROUP", "cat_w_style"),
    ("DANCE FUNDAMENTALS", "cat_w_style"),
    ("AMALGAMATION GROUP", "cat_w_style"),
    ("ENCORE", "no_style_cat"),
    ("FORMATION", "no_style_cat"),
    ("WEDDING GROUP", "no_style_cat"),
    ("PRACTICE SESSION", "no_style_cat"),
    ("ADVANCED PRACTICE SESSION", "no_style_cat"),
    ("MASTER CLASS", "no_style_cat"),
    ("CLASS", "no_style_cat"),
    ("ICE CREAM SOCIAL", "no_style_cat"),
    ("Practice Party", "no_style_cat")
]

cats = [
    "BRUNCH BUNCH",
    "RUSH HOUR GROUP",
    "FULL BRONZE GROUP",
    "ALL LEVEL GROUP",
    "GROUP",
    "DANCE SERIES",
    "BRONZE I & II GROUP",
    "DANCE FUNDAMENTALS",
    "AMALGAMATION GROUP",
    "ICE CREAM SOCIAL",
    "ENCORE",
    "FORMATION",
    "WEDDING GROUP",
    "PRACTICE SESSION",
    "ADVANCED PRACTICE SESSION",
    "MASTER CLASS",
    "COACHING",
    "MARTIN DONTIGNY COACHING",
    "PATTY CONTENTA COACHING",
    "Party"
]

styles = [
    "ARGENTINE TANGO",
    "BACHATA",
    "BOLERO",
    "CHA CHA",
    "FOXTROT",
    "LINDY HOP",
    "MERENGUE",
    "RUMBA",
    "SALSA",
    "SAMBA",
    "SWING",
    "TANGO",
    "COUNTRY TWO STEP",
    "VIENNESE WALTZ",
    "WALTZ",
    "WEST COAST SWING",
    "ZOUK",
    "FOXTROT/HUSTLE",
    "RUMBA/MERENGUE",
    "WALTZ/CHA CHA"
]

test = [
    ("ALL LEVEL", "cat"),
    ("GROUP", "cat"),
    ("WEDDING", "cat"),
    ("CHA", "dance_style"),
    ("LINDY", "dance_style")
]

cat_or_style_cl = NaiveBayesClassifier(cat_or_style_trainer,
                                       feature_extractor=custom_extractor)

cat_cl = NaiveBayesClassifier(cat_trainer,
                              feature_extractor=custom_extractor)
